@@ -42,8 +42,7 @@ Twisted. See the ``TestProtocolServer`` parser class for more details.
 Subunit includes extensions to the Python ``TestResult`` protocol. These are
 all done in a compatible manner: ``TestResult`` objects that do not implement
 the extension methods will not cause errors to be raised, instead the extension
-will either lose fidelity (for instance, folding expected failures to success
-in Python versions < 2.7 or 3.1), or discard the extended data (for extra
+will either lose fidelity, or discard the extended data (for extra
 details, tags, timestamping and progress markers).
 
 The test outcome methods ``addSuccess``, ``addError``, ``addExpectedFailure``,
@@ -121,24 +120,23 @@ import re
 import subprocess
 import sys
 import unittest
-try:
-    from io import UnsupportedOperation as _UnsupportedOperation
-except ImportError:
-    _UnsupportedOperation = AttributeError
+from io import BytesIO
+from io import UnsupportedOperation as _UnsupportedOperation
 
-from extras import safe_hasattr
-from testtools import content, content_type, ExtendedToOriginalDecorator
+import iso8601
+from testtools import ExtendedToOriginalDecorator, content, content_type
+from testtools.compat import _b, _u
 from testtools.content import TracebackContent
-from testtools.compat import _b, _u, BytesIO, StringIO
+
 try:
     from testtools.testresult.real import _StringException
     RemoteException = _StringException
 except ImportError:
     raise ImportError ("testtools.testresult.real does not contain "
         "_StringException, check your version.")
-from testtools import testresult, CopyStreamResult
+from testtools import CopyStreamResult, testresult
 
-from subunit import chunked, details, iso8601, test_results
+from subunit import chunked, details
 from subunit.v2 import ByteStreamToStreamResult, StreamResultToBytes
 
 # same format as sys.version_info: "A tuple containing the five components of
@@ -153,7 +151,27 @@ from subunit.v2 import ByteStreamToStreamResult, StreamResultToBytes
 # If the releaselevel is 'final', then the tarball will be major.minor.micro.
 # Otherwise it is major.minor.micro~$(revno).
 
-__version__ = (1, 1, 0, 'final', 0)
+__version__ = (1, 4, 4, 'final', 0)
+
+__all__ = [
+    'join_dir',
+    'tags_to_new_gone',
+    'content',
+    'content_type',
+    'TestProtocolServer',
+    'TestProtocolClient',
+    'RemoteError',
+    'RemotedTestCase',
+    'IsolatedTestCase',
+    'IsolatedTestSuite',
+    'run_isolated',
+    'TAP2SubUnit',
+    'tag_stream',
+    'ProtocolTestCase',
+    'make_stream_binary',
+    'read_test_list',
+    'TestResultStats',
+]
 
 PROGRESS_SET = 0
 PROGRESS_CUR = 1
@@ -509,9 +527,7 @@ class TestProtocolServer(object):
         """
         self.client = ExtendedToOriginalDecorator(client)
         if stream is None:
-            stream = sys.stdout
-            if sys.version_info > (3, 0):
-                stream = stream.buffer
+            stream = sys.stdout.buffer
         self._stream = stream
         self._forward_stream = forward_stream or DiscardStream()
         # state objects we can switch too
@@ -556,10 +572,10 @@ class TestProtocolServer(object):
     def _handleTime(self, offset, line):
         # Accept it, but do not do anything with it yet.
         try:
-            event_time = iso8601.parse_date(line[offset:-1])
+            event_time = iso8601.parse_date(line[offset:-1].decode())
         except TypeError:
             raise TypeError(_u("Failed to parse %r, got %r")
-                % (line, sys.exec_info[1]))
+                % (line, sys.exc_info()[1]))
         self.client.time(event_time)
 
     def lineReceived(self, line):
@@ -579,10 +595,10 @@ class TestProtocolServer(object):
     def readFrom(self, pipe):
         """Blocking convenience API to parse an entire stream.
 
-        :param pipe: A file-like object supporting readlines().
+        :param pipe: A file-like object supporting __iter__.
         :return: None.
         """
-        for line in pipe.readlines():
+        for line in pipe:
             self.lineReceived(line)
         self.lostConnection()
 
@@ -799,7 +815,7 @@ class TestProtocolClient(testresult.TestResult):
 
         ":param datetime: A datetime.datetime object.
         """
-        time = a_datetime.astimezone(iso8601.Utc())
+        time = a_datetime.astimezone(iso8601.UTC)
         self._stream.write(_b("time: %04d-%02d-%02d %02d:%02d:%02d.%06dZ\n" % (
             time.year, time.month, time.day, time.hour, time.minute,
             time.second, time.microsecond)))
@@ -810,14 +826,14 @@ class TestProtocolClient(testresult.TestResult):
         :param details: An extended details dict for a test outcome.
         """
         self._stream.write(_b(" [ multipart\n"))
-        for name, content in sorted(details.items()):
+        for name, content in sorted(details.items()):  # noqa: F402
             self._stream.write(_b("Content-Type: %s/%s" %
                 (content.content_type.type, content.content_type.subtype)))
             parameters = content.content_type.parameters
             if parameters:
                 self._stream.write(_b(";"))
                 param_strs = []
-                for param, value in parameters.items():
+                for param, value in sorted(parameters.items()):
                     param_strs.append("%s=%s" % (param, value))
                 self._stream.write(_b(",".join(param_strs)))
             self._stream.write(_b("\n%s\n" % name))
@@ -877,7 +893,8 @@ class RemotedTestCase(unittest.TestCase):
                (self._strclass(), self.__description)
 
     def run(self, result=None):
-        if result is None: result = self.defaultTestResult()
+        if result is None:
+            result = self.defaultTestResult()
         result.startTest(self)
         result.addError(self, RemoteError(_u("Cannot run RemotedTestCases.\n")))
         result.stopTest(self)
@@ -904,7 +921,8 @@ class ExecTestCase(unittest.TestCase):
         return 1
 
     def run(self, result=None):
-        if result is None: result = self.defaultTestResult()
+        if result is None:
+            result = self.defaultTestResult()
         self._run(result)
 
     def debug(self):
@@ -929,7 +947,8 @@ class IsolatedTestCase(unittest.TestCase):
     """
 
     def run(self, result=None):
-        if result is None: result = self.defaultTestResult()
+        if result is None:
+            result = self.defaultTestResult()
         run_isolated(unittest.TestCase, self, result)
 
 
@@ -943,7 +962,8 @@ class IsolatedTestSuite(unittest.TestSuite):
     """
 
     def run(self, result=None):
-        if result is None: result = testresult.TestResult()
+        if result is None:
+            result = testresult.TestResult()
         run_isolated(unittest.TestSuite, self, result)
 
 
@@ -1033,7 +1053,7 @@ def TAP2SubUnit(tap, output_stream):
             file_name=file_name, runnable=False)
     for line in tap:
         if state == BEFORE_PLAN:
-            match = re.match("(\d+)\.\.(\d+)\s*(?:\#\s+(.*))?\n", line)
+            match = re.match(r"(\d+)\.\.(\d+)\s*(?:\#\s+(.*))?\n", line)
             if match:
                 state = AFTER_PLAN
                 _, plan_stop, comment = match.groups()
@@ -1046,7 +1066,9 @@ def TAP2SubUnit(tap, output_stream):
                         file_name='tap comment')
                 continue
         # not a plan line, or have seen one before
-        match = re.match("(ok|not ok)(?:\s+(\d+)?)?(?:\s+([^#]*[^#\s]+)\s*)?(?:\s+#\s+(TODO|SKIP|skip|todo)(?:\s+(.*))?)?\n", line)
+        match = re.match(
+            r"(ok|not ok)(?:\s+(\d+)?)?(?:\s+([^#]*[^#\s]+)\s*)?"
+            r"(?:\s+#\s+(TODO|SKIP|skip|todo)(?:\s+(.*))?)?\n", line)
         if match:
             # new test, emit current one.
             _emit_test()
@@ -1074,7 +1096,7 @@ def TAP2SubUnit(tap, output_stream):
             test_name = "test %d%s" % (plan_start, description)
             plan_start += 1
             continue
-        match = re.match("Bail out\!(?:\s*(.*))?\n", line)
+        match = re.match(r"Bail out\!(?:\s*(.*))?\n", line)
         if match:
             reason, = match.groups()
             if reason is None:
@@ -1086,7 +1108,7 @@ def TAP2SubUnit(tap, output_stream):
             result = "fail"
             state = SKIP_STREAM
             continue
-        match = re.match("\#.*\n", line)
+        match = re.match(r"\#.*\n", line)
         if match:
             log.append(line[:-1])
             continue
@@ -1256,11 +1278,9 @@ def read_test_list(path):
     :param path: Path to the file
     :return: Sequence of test ids
     """
-    f = open(path, 'rb')
-    try:
-        return [l.rstrip("\n") for l in f.readlines()]
-    finally:
-        f.close()
+    with open(path, 'r') as f:
+        return [line.split('#')[0].rstrip() for line in f.readlines()
+                if line.split('#')[0]]
 
 
 def make_stream_binary(stream):
@@ -1287,15 +1307,13 @@ def _make_binary_on_windows(fileno):
 
 def _unwrap_text(stream):
     """Unwrap stream if it is a text stream to get the original buffer."""
-    if sys.version_info > (3, 0):
-        unicode_type = str
-    else:
-        unicode_type = unicode
+    exceptions = (_UnsupportedOperation, IOError)
+    unicode_type = str
     try:
         # Read streams
         if type(stream.read(0)) is unicode_type:
             return stream.buffer
-    except (_UnsupportedOperation, IOError):
+    except exceptions:
         # Cannot read from the stream: try via writes
         try:
             stream.write(_b(''))
